from optparse import OptionParser

from . import web


def parse_options():
    parser = OptionParser()

    parser.add_option(
        '--results',
        dest='results',
        default='',
        help="relative path to results folder where visual_images directory exists"
    )

    parser.add_option(
        '--baseline',
        dest='baseline',
        default=None,
        help="relative path to results folder where visual_images directory exists"
    )

    parser.add_option(
        '--host',
        dest='host',
        default='0.0.0.0',
        help="host on which server should run"
    )

    parser.add_option(
        '--port',
        dest='port',
        default=5000,
        help="port on which server should run"
    )
    opts, args = parser.parse_args()
    return parser, opts, args


def main():
    parser, opts, arguments = parse_options()

    web.start(opts.baseline, opts.results, opts.host, opts.port)



import os


HEADER = '''
    <html>
    <head>
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css" integrity="sha384-MCw98/SFnGE8fJT3GXwEOngsV7Zt27NXFoaoApmYm81iuXoPkFOJwJ8ERdknLPMO" crossorigin="anonymous">
        <link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Roboto:300,400,500,700" type="text/css">
        <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.3.1/css/all.css" integrity="sha384-mzrmE5qonljUremFsqc01SB46JvROS7bZs3IO2EmfFsd15uHvIt+Y8vEf7N7fWAU" crossorigin="anonymous">
        <script src="https://code.jquery.com/jquery-3.3.1.slim.min.js" integrity="sha384-q8i/X+965DzO0rT7abK41JStQIAqVgRVzpbzo5smXKp4YfRvH+8abtTE1Pi6jizo" crossorigin="anonymous"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.3/umd/popper.min.js" integrity="sha384-ZMP7rVo3mIykV+2+9J3UJ46jBk0WLaUAdn689aCwoqbBJiSnjAK/l8WvCWPIPm49" crossorigin="anonymous"></script>
        <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js" integrity="sha384-ChfqqxuZUCnJSK3+MXmPNIyE6ZbWh2IMqE241rYiqJxyMiZ6OW/JmZQ5stwEULTy" crossorigin="anonymous"></script>
        <script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>
        <script src="https://code.highcharts.com/highcharts.js"></script>
        <script src="http://code.highcharts.com/highcharts-3d.js"></script>
    </head>
    <body>
    <div id="piechart"></div>
    <div class="container" style="position:relative;top:20px;">
    <table class="table table-striped table-sm" id="results">
    <thead>
    <tr>
    <th></th>
    <th>Test Name</th>
    </tr>
    </thead>
    <tbody>'''


INNER_TABLE_END = '''
        </tbody>
        </table>
        </div>
        </td>
        </tr>'''

FOOTER = '''
    </tbody>
    </table>
    </div>
    <style>
      tr.accordion-toggle:hover {
        cursor:pointer;
      }
    </style>
    <script>
        $(document).ready(function() {
          var pass = 0;
          var fail = 0;
          var t = 1;
          $("table#innerResults").each(function() {
            var $items = $(this).find('tbody tr');
            $.each($items, function(n, e) {
              color = $(e).find('td:last').css('color');
              if (color == 'rgb(255, 0, 0)') {
                return false;
              }
            });
            if(color == 'rgb(255, 0, 0)') {
              $('table#results > tbody > tr:nth-child(' + t + ') > td:nth-child(2)').css('color','red');
              t = t+2;
              fail = fail + 1;
            }
            else  {
             $('table#results > tbody > tr:nth-child(' + t + ') > td:nth-child(2)').css('color','green');
             t = t+2;
             pass = pass + 1;
            }
          });
          
          $("tr.accordion-toggle").click(function () {
            classes = {
                "fa-arrow-down": "fa-arrow-right",
                "fa-arrow-right": "fa-arrow-down"
            }
            cls = $(this).find('i:first').attr("class").split(" ")[1];
            $(this).find('i:first').removeClass(cls).addClass(classes[cls]);
          });
          
          Highcharts.chart('piechart', {
            chart: {
                type: 'pie',
                options3d: {
                    enabled: true,
                    alpha: 45,
                    beta: 0
                }
            },
            title: {
                text: 'Test result stats'
            },
            tooltip: {
                pointFormat: '{series.name}: <b>{point.y:.1f}</b>'
            },
            plotOptions: {
                pie: {
                    allowPointSelect: true,
                    cursor: 'pointer',
                    depth: 35,
                    colors: ['#32CD32', '#B22222'],
                    dataLabels: {
                        enabled: true,
                        format: '<b>{point.name}</b>: {point.y:.1f}',
                        style: {
                            color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black'
                        }
                    }
                }
            },
            series: [{
                name: 'Count',
                colorByPoint: true,
                data: [{
                    name: 'Pass',
                    y: pass,
                    sliced: true,
                    selected: true
                }, {
                    name: 'Fail',
                    y: fail
                }]
            }]
          });
        });
    </script>
    </body>
    </html>'''


def make_parent_row(test_name):
    folder_name = test_name.replace(' ', '_')
    html = '''<tr data-toggle="collapse" data-target='div[value="%s"]' class="accordion-toggle">
            <td>
            <button class="btn btn-default btn-sm"><i class="fas fa-arrow-right"></i></button>
            </td>
            <td>%s</td>
            </tr>
            <tr>
            <td colspan="12" class="hiddenRow"><div class="accordian-body collapse" value="%s">
            <table class="table table-bordered table-hover" id="innerResults">
            <thead>
            <tr>
            <th style="text-align:center;vertical-align:middle">Baseline</th>
            <th style="text-align:center;vertical-align:middle">Actual</th>
            <th style="text-align:center;vertical-align:middle">Diff</th>
            <th>Diff value<br><font size="2">(Exp : Actual)</font></th>
            </tr>
            </thead>
            <tbody>''' % (folder_name, test_name, folder_name)

    return html


def make_image_row(b_path, a_path, d_path, result):
    html = '''<tr>'''

    if os.path.exists(b_path):
        html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                % (b_path, b_path)
    else:
        html += '''<td></td>'''

    if os.path.exists(a_path):
        html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                % (a_path, a_path)
    else:
        html += '''<td></td>'''

    if os.path.exists(d_path):
        html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                % (d_path, d_path)
    else:
        html += '''<td></td>'''

    html += '''<td style="color:%s;">%s</td></tr>''' % (result[0], result[1])

    return html




import os
import xml.etree.ElementTree as ET
from time import sleep

from .constants import IMAGE_EXTENSIONS

def generate_report(baseline_folder, results_folder, actual_folder=None):
    sleep(1)
    img_path = os.path.join(results_folder, 'visual_images')
    report_path = os.path.join(results_folder, 'output.xml')
    relative_baseline_folder_path = relative_path(baseline_folder, results_folder.replace(os.getcwd(), ''))
    relative_actual_folder_path = relative_path(actual_folder, results_folder.replace(os.getcwd(), '')) if actual_folder else ''

    html = '''
    <html>
    <head>
        <meta content="text/html;charset=utf-8" http-equiv="Content-Type">
        <meta content="utf-8" http-equiv="encoding">
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css" integrity="sha384-MCw98/SFnGE8fJT3GXwEOngsV7Zt27NXFoaoApmYm81iuXoPkFOJwJ8ERdknLPMO" crossorigin="anonymous">
        <link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Roboto:300,400,500,700" type="text/css">
        <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.3.1/css/all.css" integrity="sha384-mzrmE5qonljUremFsqc01SB46JvROS7bZs3IO2EmfFsd15uHvIt+Y8vEf7N7fWAU" crossorigin="anonymous">
        <script src="https://code.jquery.com/jquery-3.3.1.slim.min.js" integrity="sha384-q8i/X+965DzO0rT7abK41JStQIAqVgRVzpbzo5smXKp4YfRvH+8abtTE1Pi6jizo" crossorigin="anonymous"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.3/umd/popper.min.js" integrity="sha384-ZMP7rVo3mIykV+2+9J3UJ46jBk0WLaUAdn689aCwoqbBJiSnjAK/l8WvCWPIPm49" crossorigin="anonymous"></script>
        <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js" integrity="sha384-ChfqqxuZUCnJSK3+MXmPNIyE6ZbWh2IMqE241rYiqJxyMiZ6OW/JmZQ5stwEULTy" crossorigin="anonymous"></script>
        <script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>
        <script src="https://code.highcharts.com/highcharts.js"></script>
        <script src="http://code.highcharts.com/highcharts-3d.js"></script>
    </head>
    <body>
    <div id="piechart"></div>
    <div class="container" style="position:relative;top:20px;">
    <table class="table table-striped table-sm" id="results">
    <thead>
    <tr>
    <th></th>
    <th>Test Name</th>
    </tr>
    </thead>
    <tbody>
    '''

    tree = ET.parse(report_path)
    for t in tree.findall('.//test'):
        if not t.findall('.//kw[@name="Open Eyes"]'):
            continue

        test_name = t.get('name')
        folder_name = test_name.replace(' ', '_')

        # If the test folder does not exist, we know that it's probably a data driven test. Unless there was some other exception.
        if not os.path.exists(os.path.join(img_path, 'actual', folder_name)):
            for open_eyes_keyword in t.findall('.//kw[@name="Open Eyes"]'):
                # Get the actual test folder name from the message we log within open eyes keyword
                folder_name = open_eyes_keyword.findall('.//msg')[0].text
                if folder_name:
                    if actual_folder:
                        html = make_non_web_test_table(html, baseline_folder, relative_baseline_folder_path, img_path,
                                                       test_name, folder_name, actual_folder,
                                                       relative_actual_folder_path)
                    else:
                        html = make_test_table(html, baseline_folder, relative_baseline_folder_path, img_path, test_name,
                                        folder_name)
        else:
            if actual_folder:
                html = make_non_web_test_table(html, baseline_folder, relative_baseline_folder_path, img_path,
                                               test_name, folder_name, actual_folder, relative_actual_folder_path)
            else:
                html = make_test_table(html, baseline_folder, relative_baseline_folder_path, img_path, test_name,
                                       folder_name)
    html += '''
    </tbody>
    </table>
    </div>
    <style>
      tr.accordion-toggle:hover {
        cursor:pointer;
      }
    </style>
    <script>
        $(document).ready(function() {
          var pass = 0;
          var fail = 0;
          var t = 1;
          $("table#innerResults").each(function() {
            var $items = $(this).find('tbody tr');
            $.each($items, function(n, e) {
              color = $(e).find('td:last').css('color');
              if (color == 'rgb(255, 0, 0)') {
                return false;
              }
            });
            if(color == 'rgb(255, 0, 0)') {
              $('table#results > tbody > tr:nth-child(' + t + ') > td:nth-child(2)').css('color','red');
              t = t+2;
              fail = fail + 1;
            }
            else  {
             $('table#results > tbody > tr:nth-child(' + t + ') > td:nth-child(2)').css('color','green');
             t = t+2;
             pass = pass + 1;
            }
          });

          $("tr.accordion-toggle").click(function () {
            classes = {
                "fa-arrow-down": "fa-arrow-right",
                "fa-arrow-right": "fa-arrow-down"
            }
            cls = $(this).find('i:first').attr("class").split(" ")[1];
            $(this).find('i:first').removeClass(cls).addClass(classes[cls]);
          });

          Highcharts.chart('piechart', {
            chart: {
                type: 'pie',
                options3d: {
                    enabled: true,
                    alpha: 45,
                    beta: 0
                }
            },
            title: {
                text: 'Test result stats'
            },
            tooltip: {
                pointFormat: '{series.name}: <b>{point.y:.1f}</b>'
            },
            plotOptions: {
                pie: {
                    allowPointSelect: true,
                    cursor: 'pointer',
                    depth: 35,
                    colors: ['#32CD32', '#B22222'],
                    dataLabels: {
                        enabled: true,
                        format: '<b>{point.name}</b>: {point.y:.1f}',
                        style: {
                            color: (Highcharts.theme && Highcharts.theme.contrastTextColor) || 'black'
                        }
                    }
                }
            },
            series: [{
                name: 'Count',
                colorByPoint: true,
                data: [{
                    name: 'Pass',
                    y: pass,
                    sliced: true,
                    selected: true
                }, {
                    name: 'Fail',
                    y: fail
                }]
            }]
          });
        });
    </script>
    </body>
    </html>'''

    output_path = os.path.join(os.getcwd(), results_folder, 'visualReport.html')
    print("Creating visual report at %s" % output_path)
    output = open(output_path, 'w')
    output.write(html)
    output.close()


def relative_path(baseline_folder, results_folder):
    if baseline_folder.startswith(os.path.sep):
        baseline_folder = baseline_folder[1:]
    # This condition to avoid fixing absolute paths
    if os.path.exists(os.getcwd() + os.path.sep + baseline_folder) and not os.path.isabs(baseline_folder):
        results_folder = results_folder[1:] if results_folder.startswith(os.path.sep) else results_folder
        assert os.path.exists(results_folder)
        count = get_count_of_directories(results_folder)
        s = ''
        for i in range(count):
            s = s + ('..' + os.path.sep)
        s += baseline_folder
        return s
    else:
        # Does not exist within project root or absolute path
        return os.path.sep + baseline_folder


def get_count_of_directories(results_path):
    count = 0
    if not results_path.startswith(os.path.sep):
        results_path = os.path.sep + results_path

    for i in range(len(results_path)):
        if results_path[i] != os.path.sep and results_path[i-1] == os.path.sep:
            count += 1
    return count


def make_test_table(html, baseline_folder, relative_baseline_folder_path, img_path, test_name, folder_name):
    html += '''<tr data-toggle="collapse" data-target='div[value="%s"]' class="accordion-toggle">
            <td>
            <button class="btn btn-default btn-sm"><i class="fas fa-arrow-right"></i></button>
            </td>
            <td>%s</td>
            </tr>
            <tr>
            <td colspan="12" class="hiddenRow"><div class="accordian-body collapse" value="%s">
            <table class="table table-bordered table-hover" id="innerResults">
            <thead>
            <tr>
            <th style="text-align:center;vertical-align:middle">Baseline</th>
            <th style="text-align:center;vertical-align:middle">Actual</th>
            <th style="text-align:center;vertical-align:middle">Diff</th>
            <th>Diff value<br><font size="2">(Exp : Actual)</font></th>
            </tr>
            </thead>
            <tbody>''' % (folder_name, test_name, folder_name)

    for filename in os.listdir(baseline_folder + os.path.sep + folder_name):
        if filename.endswith('.png'):
            html += '''<tr>'''
            if os.path.exists(os.path.join(baseline_folder, folder_name, filename)):
                base_img_path = os.path.join(relative_baseline_folder_path, folder_name, filename)
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (base_img_path, base_img_path)
            else:
                html += '''<td></td>'''

            if os.path.exists(os.path.join(img_path, 'actual', folder_name, filename)):
                actual_img_path = os.path.join('visual_images', 'actual', folder_name, filename)
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (actual_img_path, actual_img_path)
            else:
                html += '''<td></td>'''

            arr = filename.split('.')
            if os.path.exists(os.path.join(img_path, 'diff', folder_name, filename)):
                diff_img_path = os.path.join('visual_images', 'diff', folder_name, filename)
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (diff_img_path, diff_img_path)
            elif os.path.exists(
                    img_path + os.path.sep + 'diff' + os.path.sep + folder_name + os.path.sep + arr[0] + '-0.png'):
                diff_img_path = 'visual_images' + os.path.sep + 'diff' + os.path.sep + \
                                folder_name + os.path.sep + arr[0] + '-0.png'
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (diff_img_path, diff_img_path)
            else:
                html += '''<td></td>'''
            txt_file = img_path + os.path.sep + 'actual' + os.path.sep + \
                       folder_name + os.path.sep + filename + ".txt"
            if os.path.exists(txt_file):
                infile = open(img_path + os.path.sep + 'actual' + os.path.sep + folder_name +
                              os.path.sep + filename + ".txt", 'r')
                first_line = infile.readline().strip()
                arr = first_line.split()
                infile.close()
                html += '''
                        <td style="color:%s;">%s</td></tr>''' % (arr[1].lower(), arr[0])
            else:
                html += '''<td></td></tr>'''

    html += '''
            </tbody>
            </table>
            </div>
            </td>
            </tr>'''
    return html


def make_non_web_test_table(html, baseline_folder, relative_baseline_folder_path, img_path, test_name,
                            folder_name, actual_folder, relative_actual_folder_path):
    html += '''<tr data-toggle="collapse" data-target='div[value="%s"]' class="accordion-toggle">
                <td>
                <button class="btn btn-default btn-sm"><i class="fas fa-arrow-right"></i></button>
                </td>
                <td>%s</td>
                </tr>
                <tr>
                <td colspan="12" class="hiddenRow"><div class="accordian-body collapse" value="%s">
                <table class="table table-bordered table-hover" id="innerResults">
                <thead>
                <tr>
                <th style="text-align:center;vertical-align:middle">Baseline</th>
                <th style="text-align:center;vertical-align:middle">Actual</th>
                <th style="text-align:center;vertical-align:middle">Diff</th>
                <th>Diff value<br><font size="2">(Exp : Actual)</font></th>
                </tr>
                </thead>
                <tbody>''' % (folder_name, test_name, folder_name)

    test_folder = os.path.join(img_path, 'actual', folder_name)

    for filename in os.listdir(test_folder):
        if filename.split('.')[-1] in IMAGE_EXTENSIONS:
            file = open(test_folder + os.path.sep + filename + '.txt', 'r')
            content = file.read()
            result, names = content.split('\n')
            file.close()

            names_arr = names.split()
            baseline_img_path = os.path.join(baseline_folder, names_arr[0])
            actual_img_path = os.path.join(actual_folder, names_arr[1])
            html += '''<tr>'''
            if os.path.exists(baseline_img_path):
                base_img_path = os.path.join(relative_baseline_folder_path, names_arr[0])
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (base_img_path, base_img_path)
            else:
                html += '''<td></td>'''

            if os.path.exists(actual_img_path):
                actual_img_path = os.path.join(relative_actual_folder_path, names_arr[1])
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (actual_img_path, actual_img_path)
            else:
                html += '''<td></td>'''

            arr = filename.split('.')
            if os.path.exists(os.path.join(img_path, 'actual', folder_name, filename)):
                diff_img_path = os.path.join('visual_images', 'actual', folder_name, filename)
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (diff_img_path, diff_img_path)
            elif os.path.exists(
                    img_path + os.path.sep + 'actual' + os.path.sep + folder_name + os.path.sep + arr[0] + '-0.png'):
                diff_img_path = os.path.join('visual_images', 'actual', folder_name) + os.path.sep + arr[0] + '-0.png'
                html += '''<td><a href="%s" target="_blank"><img src="%s" height="200" width="350"></a></td>''' \
                        % (diff_img_path, diff_img_path)
            else:
                html += '''<td></td>'''

            if result:
                arr = result.split()
                html += '''
                            <td style="color:%s;">%s</td></tr>''' % (arr[1].lower(), arr[0])
            else:
                html += '''<td></td></tr>'''

    html += '''
                </tbody>
                </table>
                </div>
                </td>
                </tr>'''
    return html


import subprocess


class Imagemagick(object):

    def __init__(self, img1, img2, diff):
        self.img1 = img1
        self.img2 = img2
        self.diff = diff

    def compare_images(self):
        compare_cmd = 'compare -metric RMSE -subimage-search -dissimilarity-threshold 1.0 "%s" "%s" "%s"' \
                      % (self.img1, self.img2, self.diff)

        attempts = 0
        while attempts < 2:
            proc = subprocess.Popen(compare_cmd,
                                    stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

            out, err = proc.communicate()
            print('Comparison output: %s' % err)
            diff = err.split()[1][1:-1]

            try:
                trimmed = float("{:.2f}".format(float(diff)))
                return trimmed
            except ValueError:
                if attempts == 0:
                    print('Comparison failed first time. Output %s' % err)
                    compare_cmd = 'magick ' + compare_cmd
                else:
                    raise Exception('Could not parse comparison output: %s' % err)
            finally:
                attempts += 1



IMAGES_FOLDER = 'visual_images'
ACTUAL_IMAGE_BASE_FOLDER = 'actual'
DIFF_IMAGE_BASE_FOLDER = 'diff'
BASELINE_IMAGE_BASE_FOLDER = 'baseline'
MODE_TEST = 'test'
MODE_BASELINE = 'baseline'
REPORT_FILE = 'visualReport.html'
REPORT_EXPIRATION_THRESHOLD = 10
IMAGE_EXTENSIONS = ['png', 'jpg', 'jpeg', 'bmp', 'raw', 'pdf']
ROBOT_LIBRARY_SCOPE = 'GLOBAL'
ROBOT_LISTENER_API_VERSION = 3


import math
import platform

from PIL import Image, ImageFilter, ImageOps
from robot.libraries.BuiltIn import BuiltIn
from selenium.common.exceptions import JavascriptException
from selenium.common.exceptions import NoSuchElementException, NoSuchFrameException


class SeleniumHooks(object):
    mobile = False

    def __init__(self, lib):
        try:
            s2l = BuiltIn().get_library_instance(lib)
            if lib == 'AppiumLibrary':
                self.driver = s2l._current_application()
                self.mobile = True
            else:
                try:
                    self.driver = s2l.driver #SeleniumLibrary v4
                except:
                    self.driver = s2l._current_browser() #SeleniumLibrary v4--
        except RuntimeError:
            raise Exception('%s instance not found' % lib)

        self.locator_strategies = {
            'xpath': self.driver.find_element_by_xpath,
            'id': self.driver.find_element_by_id,
            'class': self.driver.find_element_by_class_name,
            'css': self.driver.find_element_by_css_selector
        }

    def is_mobile(self):
        return self.mobile

    def capture_full_screen(self, path, blur=[], radius=50, redact=[]):
        self.driver.save_screenshot(path)

        if blur:
            self.blur_regions(blur, radius, path)
            if not self.is_mobile():
                initial_frame = self.driver.execute_script("return window.frameElement")
                self.driver.switch_to.default_content()
                self.blur_in_all_frames(blur, radius, path)
                self.driver.switch_to.default_content()
                print("Switching back to initial frame and name is %s" % initial_frame)
                self.driver.switch_to.frame(initial_frame)

        # User may want to blur certain elements and redact other elements at the same time.
        if redact:
            self._redact_regions(redact, path)
            if not self.is_mobile():
                initial_frame = self.driver.execute_script("return window.frameElement")
                self.driver.switch_to.default_content()
                self.redact_in_all_frames(redact, path)
                self.driver.switch_to.default_content()
                print("Switching back to initial frame and name is %s" % initial_frame)
                self.driver.switch_to.frame(initial_frame)

    def blur_in_all_frames(self, blur, radius, path):
        frames = self.driver.find_elements_by_tag_name("frame")
        iframes = self.driver.find_elements_by_tag_name("iframe")
        joined_list = frames + iframes
        print("Frames: %s" % str(len(joined_list)))
        for index, frame in enumerate(joined_list):
            print("Switching to Frame %s" % frame)
            try:
                self.driver.switch_to.frame(frame)
            except NoSuchFrameException:
                continue
            self.blur_regions(blur, radius, path)
            self.driver.switch_to.default_content()

    def redact_in_all_frames(self, redact, path):
        frames = self.driver.find_elements_by_tag_name("frame")
        iframes = self.driver.find_elements_by_tag_name("iframe")
        joined_list = frames + iframes
        print("Frames: %s" % str(len(joined_list)))
        for index, frame in enumerate(joined_list):
            print("Switching to Frame %s" % frame)
            try:
                self.driver.switch_to.frame(frame)
            except NoSuchFrameException:
                continue
            self._redact_regions(redact, path)
            self.driver.switch_to.default_content()

    def capture_element(self, path, locator, blur=[], radius=50, redact=[]):
        self.driver.save_screenshot(path)
        prefix, locator, element = self.find_element(locator)
        coord = self._get_coordinates(prefix, locator, element)
        left, right, top, bottom = self._update_coordinates(
            math.ceil(coord['left']),
            math.ceil(coord['right']),
            math.ceil(coord['top']),
            math.ceil(coord['bottom'])
        )
        self.blur_regions(blur, radius, path) if blur else ''
        self._redact_regions(redact, path) if redact else ''
        im = Image.open(path)
        im = im.crop((left, top, right, bottom))
        im.save(path)

    def capture_mobile_element(self, selector, path, blur=[], radius=50, redact=[]):
        prefix, locator, search_element = self.find_element(selector)
        location = search_element.location
        size = search_element.size
        self.driver.save_screenshot(path)
        left = location['x']
        top = location['y']
        right = location['x'] + size['width']
        bottom = location['y'] + size['height']
        self.blur_regions(blur, radius, path) if blur else ''
        self._redact_regions(redact, path) if redact else ''
        image = Image.open(path)
        image = image.crop((left, top, right, bottom))
        image.save(path)

    def scroll_to_element(self, selector):
        prefix, locator, search_element = self.find_element(selector)
        self.driver.execute_script("return arguments[0].scrollIntoView();", search_element)

    def blur_regions(self, selectors, radius, path):
        selectors = selectors if isinstance(selectors, list) else [selectors]
        for region in selectors:
            try:
                prefix, locator, element = self.find_element(region)
            except NoSuchElementException:
                continue

            left, right, top, bottom = self._get_coordinates_from_element(element)
            im = Image.open(path)
            cropped_image = im.crop((left, top, right, bottom))
            blurred_image = cropped_image.filter(ImageFilter.GaussianBlur(radius=int(radius)))
            im.paste(blurred_image, (left, top, right, bottom))
            im.save(path)

    def _redact_regions(self, selectors, path):
        selectors = selectors if isinstance(selectors, list) else [selectors]
        for region in selectors:
            try:
                prefix, locator, element = self.find_element(region)
            except NoSuchElementException:
                continue

            left, right, top, bottom = self._get_coordinates_from_element(element)
            im = Image.open(path)
            cropped_image = im.crop((left, top, right, bottom))
            readacted_image = ImageOps.colorize(cropped_image.convert('L'), black='black', white='black')
            im.paste(readacted_image, (left, top, right, bottom))
            im.save(path)

    def _get_coordinates_from_element(self, element):
        area_coordinates = self._get_coordinates_from_driver(element)

        if self.is_mobile():
            left, right = math.ceil(area_coordinates['left']), math.ceil(area_coordinates['right'])
            top, bottom = math.ceil(area_coordinates['top']), math.ceil(area_coordinates['bottom'])
        else:
            frame_abs_pos = self._get_current_frame_abs_position()
            left, right = math.ceil(area_coordinates['left'] + frame_abs_pos['x']), math.ceil(
                area_coordinates['right'] + frame_abs_pos['x'])
            top, bottom = math.ceil(area_coordinates['top'] + frame_abs_pos['y']), math.ceil(
                area_coordinates['bottom'] + frame_abs_pos['y'])

        return self._update_coordinates(left, right, top, bottom)

    def _get_current_frame_abs_position(self):
        cmd = 'function currentFrameAbsolutePosition() { \
        let currentWindow = window; \
        let currentParentWindow; \
        let positions = []; \
        let rect;  \
        while (currentWindow !== window.top) { \
            currentParentWindow = currentWindow.parent; \
            for (let idx = 0; idx < currentParentWindow.frames.length; idx++) { \
                if (currentParentWindow.frames[idx] === currentWindow) { \
                    for (let frameElement of currentParentWindow.document.getElementsByTagName("frame")) { \
                        if (frameElement.contentWindow === currentWindow) { \
                            rect = frameElement.getBoundingClientRect(); \
                            positions.push({x: rect.x, y: rect.y}); \
                        } \
                    } \
                    for (let frameElement of currentParentWindow.document.getElementsByTagName("iframe")) { \
                        if (frameElement.contentWindow === currentWindow) { \
                            rect = frameElement.getBoundingClientRect(); \
                            positions.push({x: rect.x, y: rect.y}); \
                        } \
                    } \
                    currentWindow = currentParentWindow; \
                    break; \
                } \
            } \
        } \
        return positions.reduce((accumulator, currentValue) => { \
            return { \
            x: accumulator.x + currentValue.x, \
            y: accumulator.y + currentValue.y \
            }; \
        }, { x: 0, y: 0 }); \
        }; return currentFrameAbsolutePosition();'

        try:
            coordinates = self.driver.execute_script(cmd)
        except JavascriptException:
            coordinates = {"x": 0, "y": 0}
        return coordinates

    def find_element(self, selector):
        if selector.startswith('//'):
            prefix = 'xpath'
            locator = selector
        else:
            prefix, locator = self.get_selector_parts(selector)

        strategy = self.locator_strategies[prefix]
        search_element = strategy(locator)
        return prefix, locator, search_element

    def get_selector_parts(self, selector):
        separators = [':', '=']
        prefix = locator = ''
        for separator in separators:
            selector_parts = selector.partition(separator)
            prefix = selector_parts[0].strip().lower()
            locator = selector_parts[2].strip()
            if prefix in self.locator_strategies:
                break

        else:
            if prefix not in self.locator_strategies:
                raise Exception('Unknown locator strategy %s' % prefix)
        return prefix, locator

    def _get_coordinates(self, prefix, locator, element):
        if self.mobile:
            coordinates = self._get_coordinates_from_driver(element)
        else:
            if prefix.lower() == 'xpath':
                locator = locator.replace('"', "'")
                cmd = "var e = document.evaluate(\"{0}\", document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null)" \
                      ".singleNodeValue;return e.getBoundingClientRect();".format(locator)

            elif prefix.lower() == 'css':
                locator = locator.replace('"', "'")
                cmd = "var e = document.querySelector(\"{0}\");return e.getBoundingClientRect();".format(locator)

            elif prefix.lower() == 'id':
                cmd = "var e = document.getElementById(\"{0}\");return e.getBoundingClientRect();".format(locator)

            elif prefix.lower() == 'class':
                cmd = "var e = document.getElementsByClassName(\"{0}\")[0];return e.getBoundingClientRect();" \
                    .format(locator)
            else:
                raise Exception('Invalid locator %s' % locator)

            try:
                coordinates = self.driver.execute_script(cmd)
            except JavascriptException:
                coordinates = self._get_coordinates_from_driver(element)

        return coordinates

    def _get_coordinates_from_driver(self, element):
        coordinates = {}
        location = element.location
        size = element.size
        coordinates['left'] = location['x']
        coordinates['top'] = location['y']
        coordinates['right'] = location['x'] + size['width']
        coordinates['bottom'] = location['y'] + size['height']
        return coordinates

    def _update_coordinates(self, left, right, top, bottom):
        if platform.system().lower() == "darwin":
            left = left * 2
            right = right * 2
            top = top * 2
            bottom = bottom * 2
        return int(left), int(right), int(top), int(bottom)



from datetime import datetime
import shutil
import time
from threading import Thread

from PIL import Image
from robot.libraries.BuiltIn import BuiltIn

from .constants import *
from .report_utils import *
from .imagemagick import Imagemagick
from .selenium_hooks import SeleniumHooks
from .report_generator import generate_report


class RobotEyes(object):
    output_dir = ''
    images_base_folder = ''
    test_name = ''
    baseline_dir = ''
    browser = None
    ROBOT_LISTENER_API_VERSION = ROBOT_LISTENER_API_VERSION
    ROBOT_LIBRARY_SCOPE = ROBOT_LIBRARY_SCOPE
    fail = False


    # Keeping this arg to avoid exceptions for those who have added tolerance in the previous versions.
    def __init__(self, tolerance=0):
        self.ROBOT_LIBRARY_LISTENER = self
        self.lib = 'none'

    def open_eyes(self, lib='SeleniumLibrary', tolerance=0, template_id=''):
        self.tolerance = float(tolerance)
        self.tolerance = self.tolerance / 100 if self.tolerance >= 1 else self.tolerance
        self.stats = {}
        self.fail = False
        self.baseline_dir = self._get_baseline_dir()
        self.actual_dir = self._get_actual_dir()
        self.output_dir = self._output_dir()
        self.lib = lib
        self.images_base_folder = os.path.join(self.output_dir, IMAGES_FOLDER)
        if lib.lower() != 'none':
            self.browser = SeleniumHooks(lib)
        self.test_name = BuiltIn().replace_variables('${TEST NAME}')
        if template_id:
            self.test_name += '_%s' % template_id
        test_name_folder = self.test_name.replace(' ', '_')
        print(test_name_folder)
        # delete if directory already exist. Fresh test
        self._delete_folder_if_exists(test_name_folder)
        # recreate deleted folder
        self._create_empty_folder(test_name_folder)
        self.count = 1

    # Captures full screen
    def capture_full_screen(self, tolerance=None, blur=[], radius=50, name=None, redact=[]):
        tolerance = float(tolerance) if tolerance else self.tolerance
        tolerance = tolerance/100 if tolerance >= 1 else tolerance
        name = self._get_name() if name is None else name
        name += '.png'
        path = os.path.join(self.path, name)
        self.browser.capture_full_screen(path, blur, radius, redact)
        self._fix_base_image_size(path, name)
        self.stats[name] = tolerance
        self.count += 1


    # Captures a specific region in a mobile screen
    def capture_mobile_element(self, selector, tolerance=None, blur=[], radius=50, name=None, redact=[]):
        tolerance = float(tolerance) if tolerance else self.tolerance
        name = self._get_name() if name is None else name
        name += '.png'
        path = os.path.join(self.path, name)
        self.browser.capture_mobile_element(selector, path, blur, radius, redact)
        self.stats[name] = tolerance
        self.count += 1

    # Captures a specific region in a webpage
    def capture_element(self, selector, tolerance=None, blur=[], radius=50, name=None, redact=[]):
        tolerance = float(tolerance) if tolerance else self.tolerance
        tolerance = tolerance/100 if tolerance >= 1 else tolerance
        name = self._get_name() if name is None else name
        name += '.png'
        path = os.path.join(self.path, name)
        time.sleep(1)
        self.browser.capture_element(path, selector, blur, radius, redact)
        self._fix_base_image_size(path, name)
        self.stats[name] = tolerance
        self.count += 1

    def scroll_to_element(self, selector):
        self.browser.scroll_to_element(selector)

    def compare_two_images(self, ref, actual, output, tolerance=None):
        ref += '.png' if ref.split('.')[-1] not in IMAGE_EXTENSIONS else ''
        actual += '.png' if actual.split('.')[-1] not in IMAGE_EXTENSIONS else ''
        output += '.png' if output.split('.')[-1] not in IMAGE_EXTENSIONS else ''
        tolerance = float(tolerance) if tolerance else self.tolerance
        tolerance = tolerance / 100 if tolerance >= 1 else tolerance

        if not ref or not actual:
            raise Exception('Please provide reference and actual image names')

        first_path = os.path.join(self.baseline_dir, ref)
        second_path = os.path.join(self.actual_dir, actual)

        if os.path.exists(first_path) and os.path.exists(second_path):
            diff_path = os.path.join(self.path, output)
            difference = Imagemagick(first_path, second_path, diff_path).compare_images()
            color, result = self._get_result(difference, tolerance)
            self.stats[output] = tolerance
            text = '%s %s' % (result, color)
            outfile = open(self.path + os.sep + output + '.txt', 'w')
            outfile.write(text)
            outfile.write("\n")
            img_names = '%s %s' % (ref, actual)
            outfile.write(img_names)
            outfile.close()
        else:
            raise Exception('Image %s or %s doesnt exist' % (ref, actual))

    def compare_images(self):
        test_name = self.test_name.replace(' ', '_')
        baseline_path = os.path.join(self.baseline_dir, test_name)
        actual_path = os.path.join(self.images_base_folder, ACTUAL_IMAGE_BASE_FOLDER, test_name)
        diff_path = os.path.join(self.images_base_folder, DIFF_IMAGE_BASE_FOLDER, test_name)

        if not os.path.exists(diff_path):
            os.makedirs(diff_path)

        # compare actual and baseline images and save the diff image
        for filename in os.listdir(actual_path):
            if filename.endswith('.png'):
                b_path = os.path.join(baseline_path, filename)
                a_path = os.path.join(actual_path, filename)
                d_path = os.path.join(diff_path, filename)

                if os.path.exists(b_path):
                    difference = Imagemagick(b_path, a_path, d_path).compare_images()

                    try:
                        threshold = float(self.stats[filename])
                    except ValueError:
                        raise Exception('Invalid tolerance: %s' % self.stats[filename])

                    color, result = self._get_result(difference, threshold)
                    text = '%s %s' % (result, color)
                else:
                    shutil.copy(a_path, b_path)
                    text = '%s %s' % ('None', 'green')

                output = open(actual_path + os.path.sep + filename + '.txt', 'w')
                output.write(text)
                output.close()
        BuiltIn().run_keyword('Fail', 'Image dissimilarity exceeds tolerance') if self.fail else ''

    def _delete_folder_if_exists(self, test_name_folder):
        actual_image_test_folder = os.path.join(self.images_base_folder, ACTUAL_IMAGE_BASE_FOLDER, test_name_folder)
        diff_image_test_folder = os.path.join(self.images_base_folder, DIFF_IMAGE_BASE_FOLDER, test_name_folder)

        if os.path.exists(actual_image_test_folder):
            shutil.rmtree(actual_image_test_folder)

        if os.path.exists(diff_image_test_folder):
            shutil.rmtree(diff_image_test_folder)

    def _create_empty_folder(self, test_name_folder):
        if self.lib.lower() != 'none':
            self.path = os.path.join(self.baseline_dir, test_name_folder)

            if not os.path.exists(self.path):
                os.makedirs(self.path)

        self.path = os.path.join(self.images_base_folder, ACTUAL_IMAGE_BASE_FOLDER, test_name_folder)

        if not os.path.exists(self.path):
            os.makedirs(self.path)

    def _resize(self, *args):
        for arg in args:
            img = Image.open(arg)
            img = img.resize((1024, 700), Image.ANTIALIAS)
            img.save(arg)

    def _fix_base_image_size(self, path, image_name):
        test_name = self.test_name.replace(' ', '_')
        base_image = os.path.join(self.baseline_dir, test_name, image_name)
        if os.path.exists(base_image):
            im = Image.open(path)
            width, height = im.size
            im.close()

            im = Image.open(base_image)
            b_width, b_height = im.size
            if width != b_width or height != b_height:
                im = im.resize((width, height), Image.ANTIALIAS)
                im.save(base_image)

    def _delete_report_if_old(self, path):
        t1 = datetime.fromtimestamp(os.path.getmtime(path))
        t2 = datetime.now()
        diff = (t2 - t1).seconds
        os.remove(path) if diff > REPORT_EXPIRATION_THRESHOLD else ''

    def _output_dir(self):
        output_dir = BuiltIn().replace_variables('${OUTPUT DIR}')

        if 'pabot_results' in output_dir:
            index = output_dir.find(os.path.sep + 'pabot_results')
            return output_dir[:index]
        return output_dir

    def _get_result(self, difference, threshold):
        difference, threshold = int(difference*100), int(threshold*100)
        if difference > threshold:
            color = 'red'
            result = '%s<%s' % (threshold, difference)
            self.fail = True
        elif difference == threshold:
            color = 'green'
            result = '%s==%s' % (threshold, difference)
        else:
            color = 'green'
            result = '%s>%s' % (threshold, difference)
        return color, result

    def _get_baseline_dir(self):
        baseline_dir = BuiltIn().get_variable_value('${images_dir}')
        if not baseline_dir:
            BuiltIn().run_keyword('Fail', 'Please provide image baseline directory. Ex: -v images_dir:base')

        os.makedirs(baseline_dir) if not os.path.exists(baseline_dir) else ''
        return baseline_dir

    def _get_actual_dir(self):
        return BuiltIn().get_variable_value('${actual_dir}')

    def _get_name(self):
        return 'img%s' % str(self.count)

    def _end_test(self, data, test):
        if hasattr(self, 'lib') and self.lib.lower() == 'none':
            if self.fail:
                test.status = 'FAIL'
                test.message = 'Image dissimilarity exceeds tolerance'

    def _close(self):
        if self.baseline_dir and self.output_dir:
            thread = Thread(
                target=generate_report,
                args=(self.baseline_dir, self.output_dir, self.actual_dir,)
            )
            thread.start()



import os
from optparse import OptionParser

from .report_generator import generate_report


def parse_options():
    parser = OptionParser()

    parser.add_option(
        '--baseline',
        dest='baseline',
        default=None,
        help="relative path to where baseline images directory exists"
    )

    parser.add_option(
        '--actual',
        dest='actual',
        default=None,
        help="relative path to where actual images directory exists. For non web/mobile tests"
    )

    parser.add_option(
        '--results',
        dest='results',
        default=os.getcwd(),
        help="relative path to where results directory exists"
    )

    opts, args = parser.parse_args()
    return parser, opts, args


def report_gen():
    root_folder = os.getcwd()
    parser, opts, arguments = parse_options()
    report_path = os.path.join(opts.results, 'output.xml')
    img_path = os.path.join(opts.results, 'visual_images')

    if opts.baseline is None:
        raise Exception('Please provide path to baseline image directory.')

    if os.path.exists(root_folder + os.path.sep + report_path) and os.path.exists(root_folder + os.path.sep + img_path):
        generate_report(opts.baseline, opts.results, opts.actual)
    else:
        raise Exception("Please provide a valid path to results.")



from setuptools import setup, find_packages
import os

version = '1.5.4'

setup(name='robotframework-eyes',
      version=version,
      description="Visual regression library and report generator for robot framework",
      long_description="""\
Visual regression library and report generator for robot framework. Capture elements, fullscreens and compare them against baseline images. Extends Selenium2Libary. Visit https://github.com/jz-jess/RobotEyes for documentation.""",
      classifiers=[
        'Framework :: Robot Framework',
        'Programming Language :: Python',
        'Topic :: Software Development :: Testing',
      ],
      keywords='visual-regression image-comparison robotframework robot-eyes',
      author='Jesse Zacharias',
      author_email='iamjess988@gmail.com',
      url='https://github.com/jz-jess/RobotEyes',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'pillow',
          'robotframework',
          'flask'
      ],
      entry_points={
          'console_scripts': [
              'eyes = RobotEyes.runner:main',
              'reportgen = RobotEyes.reportgen:report_gen'
          ]
      },
)



from flask import Flask, render_template, flash, redirect, url_for, request, jsonify
from gevent import pywsgi
import shutil
import os

app = Flask(__name__, static_url_path="", static_folder=os.getcwd(), template_folder='templates')

app.debug = True

app.secret_key = 'somerandomkey'


@app.route("/")
def overview():
    actual_path = os.path.join(results, 'visual_images', 'actual')
    passed = 0
    failed = 0

    if os.path.exists(actual_path):
        for directory in os.listdir(os.path.join(actual_path)):
            abs_directory = os.path.join(actual_path, directory)

            if os.path.isdir(abs_directory):
                last_found = ''  # incase we are inside an empty directory
                for file in os.listdir(abs_directory):
                    if file.endswith('.txt'):
                        f = os.path.join(abs_directory, file)
                        obj = open(f, 'r')
                        first_line = obj.readline().strip()
                        arr = first_line.split()
                        obj.close()
                        color = arr[1]

                        if color == 'red':
                            failed += 1
                            break

                        elif color == 'green':
                            last_found = 'green'
                else:
                    if last_found == 'green':
                        passed += 1

    return render_template('overview.html', passed=passed, failed=failed, total=(passed+failed))


@app.route("/passed")
def passed():
    data = {}

    actual_path = os.path.join(results, 'visual_images', 'actual')

    if os.path.exists(actual_path):
        for directory in os.listdir(os.path.join(actual_path)):
            test_name = directory.replace('_', ' ')

            abs_directory = os.path.join(actual_path, directory)

            if os.path.isdir(abs_directory):
                for file in os.listdir(abs_directory):
                    if file.endswith('.txt'):
                        f = os.path.join(abs_directory, file)

                        obj = open(f, 'r')
                        first_line = obj.readline().strip()
                        arr = first_line.split()
                        obj.close()
                        color = arr[1]

                        if color == 'red':
                            break
                else:
                    data[test_name] = directory

    return render_template('passed.html', passed=data)


@app.route("/failed")
def failed():
    data = []

    actual_path = os.path.join(results, 'visual_images', 'actual')

    if os.path.exists(actual_path):
        for directory in os.listdir(os.path.join(actual_path)):
            test_name = directory.replace('_', ' ')

            abs_directory = os.path.join(actual_path, directory)

            if os.path.isdir(abs_directory):

                for file in os.listdir(abs_directory):

                    if file.endswith('.txt'):
                        f = os.path.join(abs_directory, file)

                        obj = open(f, 'r')
                        first_line = obj.readline().strip()
                        arr = first_line.split()
                        obj.close()
                        color = arr[1]

                        if color == 'red':
                            data.append([test_name, directory])
                            break

    return render_template('failed.html', failed=data)

@app.route('/<test>')
def images(test):
    test_name = test.replace('_', ' ')
    output = {test_name: []}

    baseline_directory = os.path.join(baseline, test)
    actual_directory = os.path.join(results, 'visual_images', 'actual', test)
    diff_directory = os.path.join(results, 'visual_images', 'diff', test)

    if os.path.isdir(actual_directory):
        for file in os.listdir(actual_directory):
            if file.endswith('.png'):
                images = []
                images.append(os.path.join(baseline_directory, file))
                images.append(os.path.join(actual_directory, file))
                images.append(os.path.join(diff_directory, file))

                file = file + '.txt'
                f = os.path.join(actual_directory, file)

                if os.path.exists(f):
                    obj = open(f, 'r')
                    first_line = obj.readline().strip()
                    arr = first_line.split()
                    obj.close()
                    color = arr[1]
                    value = arr[0]
                    images.append(color)
                    images.append(value)
                    output[test_name].append(images)
    return render_template('test.html', data=output)

@app.route("/all")
def report():
    baseline_path = baseline
    actual_path = os.path.join(results, 'visual_images', 'actual')
    diff_path = os.path.join(results, 'visual_images', 'diff')

    data = {}
    if os.path.exists(actual_path):
        for directory in os.listdir(os.path.join(actual_path)):
            abs_directory = os.path.join(actual_path, directory)

            if os.path.isdir(abs_directory):
                test_name = directory.replace('_', ' ')
                data[test_name] = []

                for file in os.listdir(abs_directory):
                    if file.endswith('.png'):
                        row = []

                        b_img = os.path.join(baseline_path, directory, file)

                        row.append(b_img)
                        row.append(os.path.join(abs_directory, file))
                        row.append(os.path.join(diff_path, directory, file))

                        txt_file_name = file + '.txt'
                        txt_file_path = os.path.join(abs_directory, txt_file_name)

                        if os.path.exists(txt_file_path):
                            obj = open(txt_file_path, 'r')
                            first_line = obj.readline().strip()
                            arr = first_line.split()
                            obj.close()
                            row.append(arr)

                        row.append(directory)
                        data[test_name].append(row)
    return render_template('report.html', data=data)


@app.route("/baseline_test", methods=['POST'])
def make_all_baseline():
    tests = request.json['tests']

    baseline_path = baseline
    actual_path = os.path.join(results, 'visual_images', 'actual')

    for test in tests:
        baseline_test = os.path.join(baseline_path, test)
        actual_test = os.path.join(actual_path, test)

        if os.path.exists(baseline_test):
            shutil.rmtree(baseline_test)

        shutil.copytree(actual_test, baseline_test)

    resp = {"success": True}
    return jsonify(resp)


@app.route("/baseline_images", methods=['POST'])
def baseline_images():
    imges = request.json['images']

    for img in imges:
        actual_path = os.path.join(results, img)
        baseline_path = actual_path.replace(
            os.path.sep + 'actual' + os.path.sep, os.path.sep + 'baseline' + os.path.sep)

        if os.path.exists(actual_path):
            if os.path.exists(baseline_path):
                os.remove(baseline_path)
            shutil.copyfile(actual_path, baseline_path)

    resp = {'success': True}
    return jsonify(resp)


@app.after_request
def add_header(r):
    """
    Add headers to both force latest IE rendering engine or Chrome Frame,
    and also to cache the rendered page for 0 minutes.
    """
    r.headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
    r.headers["Pragma"] = "no-cache"
    r.headers["Expires"] = "0"
    r.headers['Cache-Control'] = 'public, max-age=0'
    return r


def start(base, res, host, port):
    global results
    global baseline
    results = res
    baseline = base
    app.logger.info('Starting server on %s:%s' % (host, port))

    try:
        pywsgi.WSGIServer((host, int(port)),
                      app).serve_forever()
    except KeyboardInterrupt:
        print('Exiting...')
